#include <avr/io.h>
#include <util/delay.h>

#include "I2C.h"


void I2C_Set_Sda(uint8_t hi) {
	if(hi) {
		I2C_DDR  &= ~I2C_SDA;
		I2C_PORT |=  I2C_SDA;
	} else {
		I2C_DDR  |=  I2C_SDA;
		I2C_PORT &= ~I2C_SDA;
	}
}

inline uint8_t I2C_Get_Sda(void) {
	return (I2C_PIN & I2C_SDA);
}

void I2C_Set_Scl(uint8_t hi) {

	_delay_loop_2(I2C_DELAY2);
	if(hi) {
		I2C_PORT |=  I2C_SCL;
	} else {
		I2C_PORT &= ~I2C_SCL;
	}
	_delay_loop_2(I2C_DELAY);

}

void I2C_Scl_Toggle(void) {
	I2C_Set_Scl(1);
	I2C_Set_Scl(0);
}

void I2C_Init(void) {
	I2C_DDR &= ~I2C_SDA;
	I2C_PORT |= I2C_SDA;
	I2C_DDR |= I2C_SCL;
}

uint8_t I2C_Write(uint8_t data) {
	uint8_t i = 9;

	while(--i) {
		if(data & 128) {
			I2C_Set_Sda(1);
		} else {
			I2C_Set_Sda(0);
		}
		data <<= 1;
		I2C_Scl_Toggle();
	}

	I2C_Set_Sda(1);
	I2C_Set_Scl(1);

	i = I2C_Get_Sda();
	I2C_Set_Scl(0);

	return (i == 0);
}

uint8_t I2C_Read(uint8_t nak) {
	uint8_t i = 9;
	uint8_t ret = 0;

	I2C_Set_Sda(1);
	I2C_Set_Scl(0);

	while(--i) {
		I2C_Set_Scl(1);
		ret <<= 1;
		if(I2C_Get_Sda()) ret |= 1;
		I2C_Set_Scl(0);
	}

	if(nak) {
		I2C_Set_Sda(1);
	} else {
		I2C_Set_Sda(0);
	}

	I2C_Scl_Toggle();
	I2C_Set_Sda(1);

	return ret;
}

uint8_t I2C_Start(uint8_t addr, uint8_t action) {
	I2C_Set_Sda(0);
	I2C_Set_Scl(0);
	return I2C_Write(action | (addr << 1));
}

void I2C_Stop(void) {
	I2C_Set_Sda(0);
	I2C_Set_Scl(1);
	I2C_Set_Sda(1);
}


